module read_eddy_m

  implicit none

contains

  subroutine read_eddy(e, d, i, hshp, ishape)

    use, intrinsic:: ISO_C_BINDING
    use, intrinsic:: ieee_arithmetic, only: ieee_value, IEEE_QUIET_NAN

    ! Libraries:
    use contour_531, only: null_polyline
    use gpc_f, only: shpobj2pol, polygon
    use jumble, only: deg_to_rad, pi
    use shapelib, only: shpobject, shpdestroyobject, shpfileisnull
    use shapelib_03, only: shp_read_object_03, dbf_read_attribute_03

    use derived_types, only: eddy, shpc_slice_handler, missing_speed

    type(eddy), intent(out):: e
    ! All components are defined. Components delta_in and delta_out
    ! get default initialization values.

    integer, intent(out):: d ! date
    integer, intent(out):: i ! eddy index
    TYPE(shpc_slice_handler), intent(in):: hshp
    integer, intent(in):: ishape ! 0-based

    ! Local:

    integer int_attr ! integer attribute
    TYPE(shpobject) psobject
    type(polygon) p
    real attr

    !---------------------------------------------------------------------

    e%cyclone = hshp%cyclone

    ! extremum:

    call dbf_read_attribute_03(e%extr%ssh, hshp%extremum, hshp%extr_ssh, &
         ishape)
    call dbf_read_attribute_03(d, hshp%extremum, hshp%extr_date, ishape)
    call dbf_read_attribute_03(i, hshp%extremum, hshp%extr_eddy_index, ishape)

    call dbf_read_attribute_03(int_attr, hshp%extremum, hshp%extr_valid, &
         ishape)
    e%valid = int_attr == 1

    call dbf_read_attribute_03(e%max_speed, hshp%extremum, hshp%extr_speed, &
         ishape)
    if (e%max_speed == missing_speed) &
         e%max_speed = ieee_value(0., IEEE_QUIET_NAN)

    call shp_read_object_03(hshp%extremum, ishape, psobject)
    e%extr%coord = [psobject%padfx(1), psobject%padfy(1)] * deg_to_rad
    call shpdestroyobject(psobject)

    ! extr_proj:
    if (.not. shpfileisnull(hshp%extr_proj)) then
       call shp_read_object_03(hshp%extr_proj, ishape, psobject)
       e%extr%coord_proj = nint([psobject%padfx(1), psobject%padfy(1)])
       call shpdestroyobject(psobject)
    end if

    ! outermost_contour:

    call dbf_read_attribute_03(attr, hshp%outermost, hshp%out_r_eq_area, &
         ishape)
    e%out_cont%area = merge(pi * attr**2 * 1e6, - 1e6, attr >= 0.)
    ! * 1e6: km2 to m2

    call dbf_read_attribute_03(e%out_cont%ssh, hshp%outermost, hshp%out_ssh, &
         ishape)

    if (hshp%out_radius4 /= - 1) then
       call dbf_read_attribute_03(e%radius4, hshp%outermost, hshp%out_radius4, &
            ishape)
    else
       e%radius4 = - 1
    end if

    call shp_read_object_03(hshp%outermost, ishape, psobject)
    p = shpobj2pol(psobject)
    call shpdestroyobject(psobject)

    if (p%nparts == 0) then
       e%out_cont%polyline = null_polyline()
    else
       e%out_cont%polyline = p%part(1)
       e%out_cont%polyline%points = e%out_cont%polyline%points * deg_to_rad
    end if

    ! max_speed_contour:

    call dbf_read_attribute_03(attr, hshp%max_speed, &
         hshp%max_speed_r_eq_area, ishape)
    e%speed_cont%area = merge(pi * attr**2 * 1e6, - 1e6, attr >= 0.)
    ! * 1e6: km2 to m2

    call dbf_read_attribute_03(e%speed_cont%ssh, hshp%max_speed, &
         hshp%max_speed_ssh, ishape)

    call shp_read_object_03(hshp%max_speed, ishape, psobject)
    p = shpobj2pol(psobject)
    call shpdestroyobject(psobject)

    if (p%nparts == 0) then
       e%speed_cont%polyline = null_polyline()
    else
       e%speed_cont%polyline = p%part(1)
       e%speed_cont%polyline%points = e%speed_cont%polyline%points * deg_to_rad
    end if

  end subroutine read_eddy

end module read_eddy_m
